/**
 * @file
 * Sort by index implementation.
 *
 * @copyright Copyright © 2012–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                      "title" : "Sort with index",
                      "keywords" : [ "sort", "indecies", "big", "high", "small", "most", ">", "<", "upper", "limit", "bound", "range" ],
                      "version" : "2.1.0",
                      "genericTypes" : {
                          "VuoGenericType1" : {
                              "defaultType" : "VuoReal",
                              "compatibleTypes" : [ "VuoInteger", "VuoReal" ]
                          }
                      },
                      "node": {
                          "exampleCompositions" : [ ]
                      }
                  });

void nodeEvent
(
        VuoInputData(VuoList_VuoReal) values,
        VuoOutputData(VuoList_VuoReal, {"name":"Sorted"}) max,
        VuoOutputData(VuoList_VuoInteger, {"name":"Index"}) position
)
{
    unsigned long valueCount = VuoListGetCount_VuoReal(values);
    VuoList_VuoReal maxOut = VuoListCreate_VuoReal();
    VuoList_VuoInteger indexOut = VuoListCreate_VuoInteger();

    for(int i = 0; i < valueCount; ++i){
        VuoInteger pos;
        VuoReal *termsArray = VuoListGetData_VuoReal(values);
        VuoListInsertValue_VuoReal(maxOut, VuoReal_max(termsArray, valueCount, &pos), 0);
        VuoListInsertValue_VuoInteger(indexOut, pos, 0);
        VuoListSetValue_VuoReal(values, 0, pos, false);
    }

    *max = VuoListSubset_VuoReal(maxOut, 1, valueCount);
    *position = VuoListSubset_VuoInteger(indexOut, 1, valueCount);

}
